#include "mex.h"

template <class T> const T& max ( const T& a, const T& b ) {
  return (b < a) ? a : b;
}

/* Usage: b = proj_omega_sub(G,H,rows,cols); */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){

  mwSize d1 = mxGetM(prhs[0]);
  mwSize d2 = mxGetM(prhs[1]);
  mwSize r = mxGetN(prhs[0]);
  
  double* G = mxGetPr(prhs[0]);
  double* H = mxGetPr(prhs[1]);

  int nb_samples = max(mxGetM(prhs[2]), mxGetN(prhs[2]));
  double* rows = mxGetPr(prhs[2]);
  
  if (max(mxGetM(prhs[3]), mxGetN(prhs[3])) != nb_samples) {
    mexErrMsgTxt("Number of rows must be equal to number of columns.");
  }
  
  double* cols = mxGetPr(prhs[3]);
  
  plhs[0] = mxCreateDoubleMatrix(nb_samples, 1, mxREAL);
  double* b = (double*) mxGetPr(plhs[0]);

  for(mwIndex s = 0; s < nb_samples; s++) {
    
    mwIndex i = (mwIndex) rows[s] - 1;
    mwIndex j = (mwIndex) cols[s] - 1;
    
    b[s] = 0;
    
    for(int k = 0; k < r; k++) {
      b[s] += G[k*d1 + i] * H[k*d2 + j];
    }
  }
}


